//////////////////////////////////////////////////////////////////////
//
// clipids: Utility functions for clipboard IDs
//          Used in ClipboardToFile and FileToClipboard
//
// (C) 2006 Claus Brod
// http://www.clausbrod.de
// Software blog: http://www.clausbrod.de/Blog/BlogOnSoftware
// 
//////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>
#include <tchar.h>

#define CLIPPAIR(c) { _T(#c), c }

struct CLIPID
{
  const TCHAR *clipformatName;
  UINT clipID;
} standardClipids[] = {
  CLIPPAIR(CF_BITMAP),
  CLIPPAIR(CF_DIB),
  CLIPPAIR(CF_DIBV5),
  CLIPPAIR(CF_DIF),
  CLIPPAIR(CF_DSPBITMAP),
  CLIPPAIR(CF_DSPENHMETAFILE),
  CLIPPAIR(CF_DSPMETAFILEPICT),
  CLIPPAIR(CF_DSPTEXT),
  CLIPPAIR(CF_ENHMETAFILE),
  CLIPPAIR(CF_GDIOBJFIRST),
  CLIPPAIR(CF_GDIOBJLAST),
  CLIPPAIR(CF_HDROP),
  CLIPPAIR(CF_LOCALE),
  CLIPPAIR(CF_METAFILEPICT),
  CLIPPAIR(CF_OEMTEXT),
  CLIPPAIR(CF_OWNERDISPLAY),  // Requires a running Win32 app
  CLIPPAIR(CF_PALETTE),
  CLIPPAIR(CF_PENDATA),
  CLIPPAIR(CF_PRIVATEFIRST),
  CLIPPAIR(CF_PRIVATELAST),
  CLIPPAIR(CF_RIFF),
  CLIPPAIR(CF_SYLK),
  CLIPPAIR(CF_TEXT),
  CLIPPAIR(CF_WAVE),
  CLIPPAIR(CF_TIFF),
  CLIPPAIR(CF_UNICODETEXT)
};

const TCHAR *FormatFromClipID(UINT clipID)
{
  for (size_t i=0; i<_countof(standardClipids); i++) {
    if (standardClipids[i].clipID == clipID)
      return standardClipids[i].clipformatName;
  }

  // Probably a registered format, let's try to get its name
  static TCHAR formatNameBuf[512];
  if (::GetClipboardFormatName(clipID, formatNameBuf, sizeof(formatNameBuf)))
    return formatNameBuf;

  return 0;
}


// ID specification     Example     Handled as
//-----------------------------------------------------
// Numerical ID         4711        Clipboard ID
//
// String represening   CF_TEXT     Standard format
// predefined format
//
// Arbitrary string     CF_FUNKY    Format to be registered
//                                  via RegisterClipboardFormat
//  
UINT ClipIDFromFormat(TCHAR *format)
{
  long id = _ttol(format);
  if (id > 0)
    return (UINT)id;

  for (size_t i=0; i<_countof(standardClipids); i++) {
    if (!_tcscmp(standardClipids[i].clipformatName, format))
      return standardClipids[i].clipID;
  }

  // none of the generic formats, register our own
  return RegisterClipboardFormat(format);
}
